---------------------------
-- lets simplify a bit..
---------------------------
local gf = Glance.Functions
local gv = Glance.Variables
local ga = Glance.Arrays
local gb = Glance.Buttons

---------------------------
-- create the button
---------------------------
gf.AddButton("Reputation","LEFT")
local btn = gb.Reputation
btn.text              = "Reputation"
btn.enabled           = true
btn.events            = {"UPDATE_FACTION"}
btn.update            = true
btn.tooltip           = true
btn.click             = true
btn.menu              = true
btn.save.perCharacter = { ["DisplayReputation"] = "Percent" }
local spc = btn.save.perCharacter
btn.save.allowProfile = true

---------------------------
-- variables
---------------------------
ga.colors["rep"] = {
	["Neutral"] = "|cffffff00",
	["Friendly"] = "|cff80ff00",
	["Honored"] = "|cff00ff00",
	["Revered"] = "|cff00ff80",
	["Exalted"] = "|cff02A5FD",
	["Unfriendly"] = "|cffff8000",
	["Hostile"] = "|cffff4000",
	["Hated"] = "|cffff0000",
}
ga.rep = {
	["3000"] = "Neutral",
	["9000"] = "Friendly",
	["21000"] = "Honored",
	["42000"] = "Revered",
	["43000"] = "Exalted",
	["0"] = "Unfriendly",
	["-3000"] = "Hostile",
	["-6000"] = "Hated"
}

---------------------------
-- update
---------------------------
function gf.Reputation.update(self, event, arg1)
	local btn = gb.Reputation
	if btn.enabled and gv.loaded then -- loaded keeps it from launching when defined
		Glance.Debug("function","update","Reputation")
		local rep = ""
		for i=1,GetNumFactions() do
			local _,_,_,bottomValue,topValue,earnedValue,_,_,_,_,_,isWatched,_ = GetFactionInfo(i)
			if isWatched then
				local val = ""
				if spc.DisplayReputation == "Percent" then
					val = gf.Reputation.get(bottomValue,earnedValue,topValue)
				else
					val = topValue - earnedValue
				end
				rep = "Rep: "..ga.colors.rep[ga.rep[""..topValue]]..val
			end
		end
		btn.button:SetText(rep)
		btn.button:SetWidth(btn.button:GetTextWidth())
	end
end

---------------------------
-- tooltip
---------------------------
function gf.Reputation.tooltip()
	Glance.Debug("function","tooltip","Reputation")
	local tooltip = gf.Tooltip
	tooltip.Title("Reputation", "GLD")
	local hasWatch = false
	for i=1,GetNumFactions() do
		local name,_,_,bottomValue,topValue,earnedValue,_,_,isHeader,_,_,isWatched,_ = GetFactionInfo(i)
		if isWatched then
			hasWatch = true
			tooltip.Line("Currently tracking "..ga.colors.rep[ga.rep[""..topValue]]..name.."|r reputation: "..ga.colors.rep[ga.rep[""..topValue]]..ga.rep[""..topValue], "WHT")
			tooltip.Space()
			tooltip.Double("Percent Earned",ga.colors.rep[ga.rep[""..topValue]]..gf.Reputation.get(bottomValue,earnedValue,topValue),"WHT","GRN")
			tooltip.Double("Points Remaining",ga.colors.rep[ga.rep[""..topValue]]..(topValue - earnedValue),"WHT","GRN")
		end
	end
	if not hasWatch then
		tooltip.Line("You are not tracking any reputation.", "WHT")
	end
	tooltip.Space()
	tooltip.Double("Display",spc.DisplayReputation,"GLD","WHT")
	tooltip.Space()
	tooltip.Line("(Click to open the Reputation tab.)","WHT")
	tooltip.Line("(Right-Click to change Reputation tracking)", "WHT")
end

---------------------------
-- menu
---------------------------
function gf.Reputation.menu(level,UIDROPDOWNMENU_MENU_VALUE)
	Glance.Debug("function","menu","Reputation")
	local HEX = ga.colors.HEX
	ExpandAllFactionHeaders()
	local hName = " "
	level = level or 1
	if (level == 1) then
		gf.setInfo(nil,true,false,true,nil,HEX.white.."Reputation Options","spacer",level,nil)
		gf.setInfo(nil,nil,true,true,nil,HEX.gold.."Display","display",level,nil)
		gf.setInfo(nil,true,false,true,nil,HEX.white.." ","spacer",level,nil)
		gf.setInfo(nil,true,false,true,nil,HEX.white.."Reputation Tracking","spacer",level,nil)
		for i=1,GetNumFactions() do
			local name,_,_,_,_,_,_,_,isHeader,_,_,_,_ = GetFactionInfo(i)
			if isHeader then
				--disabled,isTitle,hasArrow,notCheckable,checked,text,value,level,func
				gf.setInfo(nil,nil,true,true,nil,HEX.gold..name,name,level,nil)
			end
		end
	end
	if (level == 2) then
		for i=1,GetNumFactions() do
			local name,_,_,bottomValue,topValue,earnedValue,_,_,isHeader,_,_,isWatched,_ = GetFactionInfo(i)
			local val = ""
			if spc.DisplayReputation == "Percent" then
				val = gf.Reputation.get(bottomValue,earnedValue,topValue)
			else
				val = topValue - earnedValue
			end
			if isHeader then hName = name end
			if hName == UIDROPDOWNMENU_MENU_VALUE and not isHeader then
				local checked, func, text
				text = ga.colors.rep[ga.rep[""..topValue]]..ga.rep[""..topValue].." ("..val..") |r"..name
				if isWatched then
					checked = true
					func = function() SetWatchedFactionIndex(0); gf.Reputation.update() end;
				else
					checked = false
					func = function() SetWatchedFactionIndex(i); gf.Reputation.update() end;
				end
				--disabled,isTitle,hasArrow,notCheckable,checked,text,value,level,func
				gf.setInfo(nil,false,false,false,checked,text,hName,level,func)
			end
		end
		if UIDROPDOWNMENU_MENU_VALUE == "display" then
			local checked, func
			if spc.DisplayReputation=="Percent" then checked = true else checked = false end
			func = function() spc.DisplayReputation="Percent"; gf.Reputation.update() end;
			--disabled,isTitle,hasArrow,notCheckable,checked,text,value,level,func
			gf.setInfo(nil,false,false,false,checked,"Percent","Percent",level,func)
			if spc.DisplayReputation=="Remaining" then checked = true else checked = false end
			func = function() spc.DisplayReputation="Remaining"; gf.Reputation.update() end;
			--disabled,isTitle,hasArrow,notCheckable,checked,text,value,level,func
			gf.setInfo(nil,false,false,false,checked,"Remaining","Remaining",level,func)
		end
	end
end

---------------------------
-- click
---------------------------
function gf.Reputation.click(self, button, down)
	Glance.Debug("function","click","Reputation")
	if button == "LeftButton" then
		ToggleCharacter("ReputationFrame")
	end
end

---------------------------
-- get rep %
---------------------------
function gf.Reputation.get(bv,ev,tv)
	local pct
	if tv > 0 then
		pct = math.floor(((ev-bv)/(tv-bv))*100)
	else
		pct = -(math.floor(((ev-tv)/(tv-bv))*100))
	end
	return pct.."%"
end

---------------------------
-- load on demand
---------------------------
if gv.loaded then
	gf.Enable("Reputation")
end
